from datetime import datetime
from functools import wraps
from pathlib import Path
from typing import Optional, Sequence

import matplotlib.pyplot as plt
import numpy as np


def _filter_fun(x):
    return "Number of jobs available: " in x

def _map_fun(x):
    return int(
        x.split("Number of jobs available: ")[1].strip()
    )

def _get_lines_from_log_path(log_path: Path, after: Optional[str] = None):

    as_run_times = []
    nr_of_jobs_available = {}

    files = sorted(log_path.glob("*_run.log"))
    for file in files:
        with open(file, "r", encoding="utf-8") as rf:
            date = datetime.strptime(file.name.removesuffix("_run.log"), r"%Y%m%d_%H%M%S")
            if after is not None and date < after:
                continue
            as_run_times.append(date)
            lines = rf.read().splitlines()
            relevant_lines = list(filter(_filter_fun, lines))
            nr_of_jobs_available[date] = sorted(set(map(_map_fun, relevant_lines)))
    
    return as_run_times, nr_of_jobs_available

def asrun_with_jobs_available(
    func,
    log_path: Path,
    after: Optional[datetime] = None,
    rel_y_position: float | Sequence[float] = 0.8,
    color_text: str = "#BBBBBB",
    color_line: str = "#BBBBBB",
):
    @wraps(func)
    def plotter(*args, **kwargs):
        local_rel_y_position = rel_y_position
        if isinstance(local_rel_y_position, float):
            local_rel_y_position = [local_rel_y_position]
        as_run_times, nr_of_jobs_available = _get_lines_from_log_path(log_path, after)
        fig = func(*args, **kwargs)
        ax = fig.get_axes()[0]
        y0, y1 = ax.get_ylim()
        local_rel_y_position = (y1 - y0) * np.array(local_rel_y_position) + y0
        for as_run_time in as_run_times:
            ax.axvline(as_run_time, color=color_line, ls="-", zorder=-1)
            nr_jobs = nr_of_jobs_available[as_run_time]
            text = (
                f"Num. jobs: {', '.join(str(element) for element in nr_jobs)}"
                if nr_jobs
                else ""
            )
            for yy in local_rel_y_position:
                ax.text(as_run_time, yy, text, color=color_text, rotation="vertical", zorder=-1)
        return fig
    return plotter